; Convert a XML file into a flat table.

(ns xml2table
  (:require [clojure.java.io :as io]
            [clojure.xml :as xml]
            [clojure.string :as str]))

(defn find-tag
  "In the XML tree, find the tag specified as a keyword."
  [tree tag]
  (let [tags (map :tag (tree :content))
        index (.indexOf tags tag)]
    (get (tree :content) index)))

(defn find-path
  "Traverse the XML tree using the list of tags."
  [tree tags]
  (let [tag (first tags)
        new-tree (find-tag tree tag)]
    (if (or (empty? (rest tags)) (nil? new-tree))
      new-tree
      (recur new-tree (rest tags)))))

(defn path-to-tags
  "Convert a string path in the form A/B/C to a list of tags (:A :B :C)"
  [path]
  (map keyword (str/split path #"/")))

(defn keys-to-strings
  "Convert the keyword keys to string."
  [table]
  (map (fn [row]
         (apply hash-map (flatten (for [[k v] row]
           [(name k) v]))))
       table))

(defn to-table
  "Convert the tree contents to a flat table.
  This takes the attributes of all tags."
  [tree]
  (keys-to-strings (map (fn [row] (assoc (:attrs row) :tag (name (:tag row))))
                        (tree :content))))

(defn parse-xml-string
  "Parse a string as XML."
  [s]
  (let [clean-s (str/replace (.trim s) #"[^\x20-\x7e\x0A]" "")]
    (xml/parse (java.io.ByteArrayInputStream. (.getBytes clean-s)))))

(defn flatten-xml
  "Flatten the XML document into a table by selecting a child node.
  The path is in the form of A/B/C."
  [xml path]
  (if (empty? path)
    (vec (to-table (parse-xml-string xml)))

    (vec (to-table (find-path (parse-xml-string xml) (path-to-tags path))))))

(defn read-file
  "Read in a file and return its contents as a string."
  [f]
  (if (string? f)
    (slurp f)
    ""))
